/****************************************************************
 *								*
 *	DDT.H		include file for COMSET suite		*
 *								*
 *   the definition in C terms of the structure used to define	*
 *   the port controls in the assembler code			*
 *								*
 ****************************************************************/

struct ddt {
	int	 ddt_base;	    /* port base address */
	char	 ddt_intr;	    /* interupt request line to be used */
	char	 ddt_sub_addr;	    /* port sub address */
	unsigned ddt_baud;	    /* divider to produce requested baud rate */
	int	 ddt_mode;	    /* flags (see definitions below) */
	int	 ddt_line;	    /* initial line control bits */
	unsigned ddt_tx_size;	    /* size in bytes of the transmit buffer */
	unsigned ddt_rx_size;	    /* size in bytes of the receiver buffer */
	int	 ddt_force;	    /* buffer to hold XON/XOFF protocol char */
	unsigned ddt_tx_count;	    /* number of bytes in the transmit buffer */
	unsigned ddt_rx_count;	    /* number of bytes in the receiver buffer */
	unsigned ddt_tx_seg;	    /* segment of tx buffer start */
	unsigned ddt_rx_seg;	    /* segment of rx buffer start */
	unsigned ddt_tx_off;	    /* offset of tx buffer start */
	unsigned ddt_rx_off;	    /* offset of rx buffer start */
	unsigned ddt_tx_pointer;    /* pointer to next byte in tx buffer */
	unsigned ddt_rx_pointer;    /* pointer to next byte in rx buffer */
	char	 ddt_ier_mask;	    /* value expected in the IER */
	char	 ddt_pic_mask;	    /* byte pic mask */
	unsigned ddt_time_out_tx;   /* word timeout delay transmit */
	unsigned ddt_time_out_rx;   /* word timeout delay receive */
	unsigned ddt_rx_xon;	    /* space for sending xon */
	unsigned ddt_rx_xoff;	    /* space to send xoff */
	char	 ddt_modem;	    /* saved modem control reg bits */
	char	 ddt_emode;	    /* extended mode */
	unsigned ddt_debug;
};

/*
 *  bits used in MODE int
 */

#define M_XTX	000001	/* generate Xon/Xoff protocols */
#define M_XRX	000002	/* honour any Xon/Xoff protocols received */
#define M_CTS	000004	/* wait the sender till CTS is active */
#define M_RTS	000010	/* remove RTS when the transmitter que is empty */
#define M_CD	000020	/* wait the sender till CD is active */
#define M_NORM	000040	/* normal PC/XT/AT mode (over-rides all else) */
#define M_DSR	000100	/* wait sender til DSR active */
#define M_DTR	000200	/* remove DTR when receiver is nearly full */


#define M_XSET	000400	/* we have been told to Xoff */
#define M_XSENT	001000	/* we have sent Xoff */
#define M_SETUP	002000	/* set by COMS_INIT,subsequent calls will only */

#define M_TXTO	004000	/* inhibit TX timeouts */
#define M_RXTO	010000	/* inhibit RX timeouts */
#define M_ERROR	020000	/* error logging mode on */
#define M_ECHO  040000	/* echo RX to TX */
#define M_FIFO  0100000	/* UARTS with FIFOs detected */

#define M_TMS	000001	/* TMS enq/ack fix */
#define M_FORCE 000002	/* Force acceptance of a port address */

/*
 *  bits used in LINE
 */

#define DB5	000	/* 5 data bits */
#define DB6	001	/* 6 data bits */
#define DB7	002	/* 7 data bits */
#define DB8	003	/* 8 data bits */

#define OSB	000	/* one stop bit */
#define TSB	004	/* two stop bits */

#define NOPAR	000	/* no parity generation or checking */
#define PARE	030	/* use even parity */
#define PARO	010	/* use odd parity */
#define PAR1	070	/* always send parity as 1 */	
#define PAR0	050	/* always send parity as 0 */


/*
 *   the device headers in COMS.SYS are according to the spec
 *   in the PCDOS manual ie:
 */

struct header {
		long header_link;	/* far address of next header */
		int header_mode;	/* mode flags */
		unsigned header_strat;	/* strategy routine offset */
		unsigned header_intr;	/* interupt routine offset */
		char header_name[8];	/* device name */
};

/*
 *  these are the offsets for where a struct is not easy to use
 */

#define BASE		0	/* port base address */
#define	INTR		2	/* interupt request line to be used */
#define SUBADDR		3	/* port subaddress */
#define	BAUD		4	/* divider to produce requested baud rate */
#define	MODE		6	/* flags (see definitions below) */
#define	LINE		8	/* initial line control bits */
#define	TX_SIZE		10	/* size in bytes of the transmitter buffer */
#define	RX_SIZE		12	/* size in bytes of the receiver buffer */
#define	FORCE		14	/* buffer to hold XON/XOFF protocol char */
#define	TX_COUNT	16	/* number of bytes in the transmitter buffer */
#define	RX_COUNT	18	/* number of bytes in the receiver buffer */
#define	TX_SEG		20	/* segment of tx buffer start */
#define	RX_SEG		22	/* segment of rx buffer start */
#define	TX_OFF		24	/* offset of tx buffer start */
#define	RX_OFF		26	/* offset of rx buffer start */
#define	TX_POINTER	28	/* pointer to next byte in tx buffer */
#define	RX_POINTER	30	/* pointer to next byte in rx buffer */
#define IER_MASK	32	/* value expected in the IER */
#define PIC_MASK	33	/* byte pic mask */
#define	TIME_OUT_TX	34	/* word timeout delay transmit */
#define	TIME_OUT_RX	36	/* word timeout delay receive */
#define RX_XON		38	/* space for Xon */
#define RX_XOFF		40	/* space for Xoff */

#define DDT_SIZE	sizeof(struct ddt)	/* number of bytes in DDT */
