	page	60,132
;
; ***********************************************************************
; *									*
; *   RX	(c) Combro		02/11/85	created		*
; *					19/01/87	amended		*
; *					02/06/87	re-amended	*
; *					18/12/87	add /n		*
; *		by  Nigel V.Hewitt BSc.					*
; *									*
; ***********************************************************************
;
;   RX started as a "bare-bones" of a utility to copy files or other
;   data from one system to a file on this system. You call
;   RX with a valid file pathname as the first parameter and the
;   required port number (1-10) as the second and all the data
;   read from the serial port is copied to a buffer. When
;   the ESC key is pressed the data collection is ended and the
;   buffer contents are written to the named file.
;
;   RX has grown to accommodate Combro's internal needs with an activity
;   monitor, so you can see that data is coming, and the /n option
;   to type the last keystroke to make the 'host' send the file. We use
;   it with our UNIX box, typing    'cat filename' into RS232, then
;   toggling to DOS and running RX to grab the file. We get a trailing $
;   in the file but this edits out easily enough.
;
;   I suggest you read the code for MODEM.ASM prior to this
;   as many of the ideas are discussed in greater detail there
;
;   to assemble this file using current generation MASM and LINK
;
;	masm rx;		assembles
;	link rx/e;		links with exepacking to keep it small
;
;
; constants that control how this code assembles
;
ncom		equ	10	; maximum port is currently com10:
b_size		equ	4096	; buffer size (multiple of sector size)
init		equ	0	; 1= initialise port / 0 = do not
;
;
; useful constants
;
tab		equ	09h	; ascii tab
cr		equ	0dh	; ascii carriage return
lf		equ	0ah	; ascii line feed
escp		equ	1bh	; ascii escape character
;
	page
;
data		segment
com		dw	0		; com device to use
psb_seg		dw	?		; segment address of the psb
is_open		db	0		; flag that file has been opened
file_name	db	65 dup(0)	; asciiz filename
ten		db	10		; a useful constant
handle		dw	?		; dos file handle
buffer		db	b_size dup (?)	; storage buffer
enter_flag	db	0		; flag to send enter
;
;
; text messages
;
sign_on	db	cr,lf,'RX : copies from a com port to a file'
	db	cr,lf,'by Combro 01 Jan 88'
	db	cr,lf,lf,'Key ESC to terminate session'
	if	init eq 1
	db	cr,lf,'Port is initialised to 1200 baud,'
	db	'8 bit no parity'
	else
	db	cr,lf,'Port is expected to be previously'
	db	' initialised'
	endif
	db	cr,lf,'$'
ent_key	db	'Enter key will be sent',cr,lf,'$'
bye_bye	db	cr,lf,lf,'RX Terminating$'
error	db	cr,lf,'RX Parameter errors'
	db	cr,lf,'Enter   RX pathname,p[/n]'
	db	cr,lf,'Where pathname is the file you want data to go to'
	db	cr,lf,'And   p        is the number of the com port'
	db	cr,lf,'And   /n       is optional "send an enter" flag'
	db	cr,lf,'$'
errm_1	db	cr,lf,'File name not correctly specified$'
errm_2	db	cr,lf,'Port number not specified$'
errm_3	db	cr,lf,'Port number not in range$'
errm_4	db	cr,lf,'File will not open$'
errm_5	db	cr,lf,'Optional parameter error$'
;
active	db	cr,'Active $'
silent	db	cr,'Silent $'
run	db	0		; flag set on data comming
old_run	db	1
off_0	db	0
data	ends	
;
	page
;
code	segment
;
	assume	cs:code,ds:data
;
;
; this is where things start. I save the ES register
; as it contains a pointer to the PSB segment with the command tail
;
rx	proc	far
	sti				; interrupts on
	mov	ax,seg data		; set up segment register
	mov	ds,ax
	mov	ax,es			; pointer to PSB segment
	mov	psb_seg,ax		; save it
;
;
; we start by announcing ourself
;
	mov	dx,offset sign_on	; a pointer to the string
	mov	ah,9			; text to screen
	int	21h			
;
;
; now find the pathname to use by copying its name from the
; command tail in the PSB (still pointed to by ES:)
;
	mov	cl,byte ptr es:80h	; get byte count of parameters
	xor	ch,ch
	jcxz	k2a			; if zero don't loop
	mov	si,81h			; point to first real character
	mov	di,offset file_name	; where we want the file name
k1:	mov	al,es:[si]		; get a character
	inc	si			; bump pointer
	cmp	al,' '			; reject leading spaces
	je	k2
	cmp	al,tab			; and tabs
	jne	k4			; move on on real characters
k2:	loop	k1			; loop on byte count
k2a:	jmp	err1			; run out of input
;
k3:	mov	al,es:[si]		; get the next character
	inc	si			; bump the pointer
k4:	cmp	al,' '			; filenames have no spaces...
	je	k6
	cmp	al,','			; or commas...
	je	k6
	mov	[di],al			; save in file name
	inc	di
	loop	k3			; go for more
	jmp	err2			; run out and no port number	
;
; get the port number
;
k5:	mov	al,es:[si]		; get the next
	inc	si
k6:	cmp	al,'0'			; looking for digits now
	jb	k7			; to small
	cmp	al,'9'
	jle	k9			; ok
k7:	loop	k5			; loop till we find some
	jmp	err2			; no port number
;
k8:	mov	al,es:[si]		; get the next digit
	inc	si
	cmp	al,'0'			; test if still digits
	jb	k10			; finished
	cmp	al,'9'
	ja	k10
k9:	sub	al,'0'			; convert to binary value
	mov	dl,al			; hold current digit in DX
	xor	dh,dh
	mov	ax,com			; get story so far
	mul	ten			; *10 as we have another digit
	add	ax,dx			; add this digit
	mov	com,ax			; save new result
	loop	k8			; go for more, if any
k10:	mov	ax,com			; get the desired port 1-10	
	dec	ax			; convert 1-10 to 0-9
	cmp	ax,ncom			; test the final result is legal
	jb	k11a			; jump 'near' not 'short'
	jmp	err3
k11a:	mov	com,ax			; set this as the port to use
;
; test for optional /n switch
;
	jcxz	s7			; no more string
	dec	si			; return to delimiter
s1:	mov	al,es:[si]		; read a character
	inc	si			; point to next
	cmp	al,' '
	je	s2			; move over spaces
	cmp	al,tab
	jne	s3
s2:	loop	s1
	jmp	short s7
s3:	cmp	al,'/'			; demand a '/'
	jne	s4			; error
	mov	al,es:[si]
	cmp	al,'n'
	je	s6
	cmp	al,'N'
	je	s6
s4:	jmp	err5			; report error
s6:	mov	enter_flag,1
	mov	dx,offset ent_key
	mov	ah,9			; text to screen
	int	21h
s7:	mov	dx,offset active	; put up the 'active' text
	mov	ah,9
	int	21h
;
; create and open the file (deleting any previous)
;
	mov	dx,offset file_name	; point to file name
	mov	cx,0			; attribute for normal file
	mov	ah,3ch			; file create service
	int	21h			; dos entry
	jc	s7b			; fails to create
	mov	dx,offset file_name	; point to file name
	mov	al,1			; open for writing
	mov	ah,3dh			; file open service
	int	21h			; dos entry
	jnc	s7b			; ok	
s7a:	jmp	err4			; fails to open
s7b:	mov	handle,ax		; save the file handle
;
; if initialisation set the port up
;
	if	init eq 1
	mov	dx,com			; required port
	mov	ah,0
	mov	al,83h			; 1200 baud,8 bit,no-parity
	int	14h
	endif
	xor	di,di			; set buffer start
;
; if the enter_flag is set send an enter
;
	cmp	enter_flag,0
	je	s8
	mov	dx,com			; this port
	mov	ah,1			; send a character
	mov	al,cr			; cr = enter
	int	14h
s8:
;
;
; start of loop hunting for keys and data from the line
;
k11:	sti				; interrupts must be on
	mov	ah,1			; request the keyboard status
	int	16h
	jz	k12			; no characters waiting
	mov	ah,0			; request the character
	int	16h
	cmp	al,escp			; if it's ESC we exit
	je	k13
	jmp	short k11		; loop
;
; no key waiting so test the com line
;
k12:	mov	dx,com			; com?: port number
 	mov	ah,3
	int	14h			; get the serial status
	test	ah,1			; test for Data AVailable (DAV) ?
	jz	k12a			; none so loop
	mov	run,1			; note that data is running
	cmp	old_run,0		; are we saying `silent' ?
	je	k12b			; yes, so change that
	mov	ah,2			; read the data from the port
	int	14h
	mov	buffer[di],al		; save the character
	inc	di			; next space
	cmp	di,b_size		; test DI
	jne	k11			; not maximum yet
;
; we have run to the end of the buffer so we had better
; save the current buffer full and go back for more or data
; will be lost. Here the Combro drivers begin to pay real dividends
; as while we write this disk file data will accumulate in the
; buffers (assuming you have not been to stingy in their sizes) and be
; waiting for us when we return.
;
	call	write		; write the data to the file
	xor	di,di		; rewind the pointer
	jmp	short k11
;
;
; test for activity : about every second or when data starts
;
; every 16 clock ticks the clock tick counter will have its bottom
; 4 bits at zero. When this is so I test the 'run' flag which is
; set by received data and display `Active' or `Silent' as needed.
; To save time I do not overwrite if there is no change.
; We started with `Active' ie: old_run == 1
;
k12a:	mov	ah,0		; get time in CX:DX
	int	1ah		; bios call
	and	dl,0fh		; bottom 4 bits == 0 ?
	jz	k12aa
	mov	off_0,1		; enable retry
	jmp	k11
;
k12aa:	cmp	off_0,0		; we are not on the same tick are we ?
	je	k11
	mov	al,run		; has activity state changed ?
	mov	run,0		; clear flag
	cmp	al,old_run
	je	k12d		; no, so exit
	jmp	short k12bb
k12b:	mov	al,run		; also enter here from unexpected data
k12bb:	mov	old_run,al	; save as old activity flag
	or	al,al		; activity ?
	jz	k12c
	mov	dx,offset active
	jmp	short k12c
k12c:	mov	dx,offset silent
	mov	ah,9
	int	21h
k12d:	mov	off_0,0		; clear the off zero flag
	jmp	k11
;
; operator has selected escape so save the data to a file
; we must beware of the case of exactly one bufferfull being
; collected. (this is not as unlikely as it sounds, the disk activity
; during the file write is an effective "that's enough" operator
; prompt...)
;
k13:	or	di,di		; is there any data to save ?
	jz	k14		; not now
	call	write		; write the data to the file
;
k14:	mov	bx,handle	; get the file handle
	mov	ah,3eh		; close a file service
	int	21h
;
; sign out and return with errorlevel 0
;
	mov	dx,offset bye_bye	; sign off
	mov	ah,9			; print a string
	int	21h
;
	mov	ah,4ch			; return to DOS
	mov	al,0			; set errorlevel 0
	int	21h
;
; error exit : the operator has not entered a valid
; command line so we tell him what to do
;
err1:	mov	dx,offset errm_1	; no file name
	jmp	short e1
err2:	mov	dx,offset errm_2	; no port number
	jmp	short e1
err3:	mov	dx,offset errm_3	; port number to big
	jmp	short e1
err4:	mov	dx,offset errm_4	; file won't open
	jmp	short e1
err5:	mov	dx,offset errm_5	; problem with flag
e1:	mov	ah,9
	int	21h
	mov	dx,offset error		; try to be generally helpful
	mov	ah,9			; print a string
	int	21h
	mov	ah,4ch			; return to DOS
	mov	al,1			; set errorlevel 1
	int	21h
rx	endp			
;
;
;
; write to the current data buffer to the file
;
write	proc	near
	mov	bx,handle		; file handle
	mov	dx,offset buffer	; start of data buffer
	mov	cx,di			; contents of data buffer
	mov	ah,40h			; write to file service
	int	21h			; dos entry
	ret
write	endp
code	ends
;
;
sseg	segment	para stack
	dw	100 dup (?)
sseg	ends
;
	end	rx		; start of execution address
